DIN = 3
DOUT = 4
DIMM = 5
TIMER = 6
BUTTON = 7
LEDRGB = 9
LEDRGBW = 9
GSM = 10
ROLLER_SH = 11
ANALOGIN = 12
POWERSUPPLYVOLTAGE = 12
POWER_SUPPLY_VOLTAGE = 12
ANALOGOUT = 13
PIDCONTROLLER = 14
CALENDAR = 15
SCHEDULER = 16
GENERIC_ZW_NODE = 17
THERMOSTAT = 18
PANEL_PAGE = 19
TEMPERATURE_SENSOR = 20
PANELSENSTEMP = 20
PANELSENSLIGHT = 21
HUMIDITY_SENSOR = 22
ONEW_SENSOR = 23
ROLLER_SHUTTER = 24
WAKE_UP = 24
ZWAVE_BINARY_SENSOR = 25
BINARY_SENSOR = 25
DOUT_L = 26
DALI_MASTER = 27
PANEL = 29
PANEL_BUTTON = 30
PUSH = 31
DALI_GEAR = 32
PRESSURE_SENSOR = 34
DALI_GEAR_DT8 = 35
AIR_VOC_SENSOR = 36
SOUND_SENSOR = 37
IR_CONTROLLER = 38
SUNRISESUNSETCALENDAR = 39
PRESENCESENSOR = 40
AIR_CO2_SENSOR = 41
LIGHT_SENSOR_LUX = 42
EVENTSCHEDULER = 43
ZWAVE_THERMOSTAT = 51
ZWAVE_DOUT = 52
ZWAVE_DIN = 53
ZWAVE_WAKEUP = 54
ZWAVE_BATTERY = 55
ZWAVE_TAMPER_ALARM = 56
ZWAVE_POWER_METER = 57
ZWAVE_ANALOG_SENSOR = 58
ZWAVE_ROLLER_SHUTTER = 59
ZWAVE_DIMMER = 60
ZWAVE_UNKNOWN_MODULE = 61
ZWAVE_RGBW_LED = 62
ZWAVE_1W_SENSOR = 63
ZWAVE_CONFIG = 64
ZWAVE_FAKRO = 65
ZWAVE_FAKRO_ZWS = 65
ZWAVE_IR = 66
ZWAVE_IR_EP = 67
ZWAVE_WEATHER = 68
SATELZONE = 100
ZONE = 100
INPUT = 101
SATELINPUT = 101
OUTPUT = 102
SATELOUTPUT = 102
SATEL = 103
HTTPLISTENER = 120
HTTPREQUEST = 121
JABLOTRON = 150
JABLOTRONSECTION = 151
JABLOTRONOUTPUT = 152
JABLOTRONDEVICE = 153
MODBUS = 200
MODBUSVALUE = 201
COOLMASTERNET = 250
COOLMASTER = 252
SONOS = 260
MUSICCAST = 261
